@@ -1,7 +1,6 @@
 #!/bin/sh
-# Copyright (c) 2021 Tailscale Inc & AUTHORS All rights reserved.
-# Use of this source code is governed by a BSD-style
-# license that can be found in the LICENSE file.
+# Copyright (c) Tailscale Inc & AUTHORS
+# SPDX-License-Identifier: BSD-3-Clause
 #
 # This script detects the current operating system, and installs
 # Tailscale according to that OS's conventions.
@@ -103,7 +102,7 @@ main() {
 					APT_KEY_TYPE="keyring"
 				fi
 				;;
-			parrot)
+			parrot|mendel)
 				OS="debian"
 				PACKAGETYPE="apt"
 				if [ "$VERSION_ID" -lt 5 ]; then
@@ -114,6 +113,18 @@ main() {
 					APT_KEY_TYPE="keyring"
 				fi
 				;;
+			galliumos)
+				OS="ubuntu"
+				PACKAGETYPE="apt"
+				VERSION="bionic"
+				APT_KEY_TYPE="legacy"
+				;;
+			pureos)
+				OS="debian"
+				PACKAGETYPE="apt"
+				VERSION="bullseye"
+				APT_KEY_TYPE="keyring"
+				;;
 			raspbian)
 				OS="$ID"
 				VERSION="$VERSION_CODENAME"
@@ -172,7 +183,7 @@ main() {
 				VERSION=""
 				PACKAGETYPE="dnf"
 				;;
-			rocky|almalinux|nobara)
+			rocky|almalinux|nobara|openmandriva|sangoma)
 				OS="fedora"
 				VERSION=""
 				PACKAGETYPE="dnf"
@@ -212,6 +223,11 @@ main() {
 				VERSION="$VERSION_ID"
 				PACKAGETYPE="apk"
 				;;
+			postmarketos)
+				OS="alpine"
+				VERSION="$VERSION_ID"
+				PACKAGETYPE="apk"
+				;;
 			nixos)
 				echo "Please add Tailscale to your NixOS configuration directly:"
 				echo
@@ -337,7 +353,9 @@ main() {
 			fi
 		;;
 		amazon-linux)
-			if [ "$VERSION" != "2" ]
+			if [ "$VERSION" != "2" ] && \
+			   [ "$VERSION" != "2022" ] && \
+			   [ "$VERSION" != "2023" ]
 			then
 				OS_UNSUPPORTED=1
 			fi
